﻿Option Strict On
Option Explicit On

Imports System.Runtime.InteropServices

Public Class IMAPI2

#Region "API"
    <DllImport("Ole32.dll", EntryPoint:="CoCreateInstance")>
    Private Shared Function CoCreateInstance(<[In]> ByRef rclsid As Guid,
                                             <[In]> pUnkOuter As IntPtr,
                                             <[In]> dwClsContext As Integer,
                                             <[In]> ByRef riid As Guid,
                                             <Out> ByRef ppv As IntPtr) As Integer
    End Function

#End Region

#Region "Enums"
    Private Enum VTable_Interfaces As Integer

        ' ---=== IUnknown ===---
        ' QueryInterface = 0
        ' AddRef = 1
        ' Release = 2

        ' ---=== IDispatch ===---
        ' GetTypeInfoCount = 3
        ' GetTypeInfo = 4
        ' GetIDsOfNames = 5
        ' Invoke = 6

        ' ---=== IDiscMaster2 ===---
        GetItem = 8
        GetCount = 9
        GetIsSupportedEnvironment = 10

        ' ---=== IDiscRecorder2 ===---
        InitializeDiscRecorder = 13
        GetVendorId = 15
        GetProductId = 16
        GetProductRevision = 17
        GetVolumeName = 18
        GetVolumePathNames = 19

        ' ---=== IDiscFormat2 ===---
        IsRecorderSupported = 7
        IsCurrentMediaSupported = 8
        GetMediaPhysicallyBlank = 9
        GetMediaHeuristicallyBlank = 10

        ' ---=== IDiscFormat2TrackAtOnce ===---
        PrepareMedia = 12
        ReleaseMedia = 15
        PutRecorder = 17
        GetTotalSectorsOnMedia = 22
        GetFreeSectorsOnMedia = 23
        GetUsedSectorsOnMedia = 24
        PutClientName = 29

    End Enum
#End Region

#Region "Const"
    Private Const S_OK As Integer = &H0
    Private Const CLSCTX_INPROC_SERVER As Integer = &H1

    Private Const MsftDiscMaster2 As String = "2735412e-7f64-5b0f-8f00-5d77afbe261e"
    Private Const IID_IDiscMaster2 As String = "27354130-7f64-5b0f-8f00-5d77afbe261e"

    Private Const MsftDiscRecorder2 As String = "2735412d-7f64-5b0f-8f00-5d77afbe261e"
    Private Const IID_IDiscRecorder2 As String = "27354133-7f64-5b0f-8f00-5d77afbe261e"

    Private Const MsftDiscFormat2TrackAtOnce As String = "27354129-7f64-5b0f-8f00-5d77afbe261e"
    Private Const IID_IDiscFormat2TrackAtOnce As String = "27354154-8f64-5b0f-8f00-5d77afbe261e"

#End Region

#Region "Class"
    Sub New()
        GetCdDriveInfos()
    End Sub
#End Region

#Region "Variable"
    Private ReadOnly m_CdDriveInfos As New List(Of CdDriveInfo)
    Private m_RecorderUniqueId As String = String.Empty
#End Region

#Region "Structure"
    Private Structure CdDriveInfo
        Dim UniqueId As String
        Dim VendorId As String
        Dim ProductId As String
        Dim ProductRevision As String
        Dim VolumeName As String
        Dim VolumePathNames As List(Of String)
        '.... was man sonst noch so an Infos vom CD-Laufwerk haben möchte
    End Structure
#End Region

#Region "Properties"
    Public ReadOnly Property GetCdDriveCount As Integer
        Get
            Return m_CdDriveInfos.Count
        End Get
    End Property

    Public ReadOnly Property GetCdDriveUniqueId(index As Integer) As String
        Get
            Return m_CdDriveInfos(index).UniqueId
        End Get
    End Property

    Public ReadOnly Property GetCdDriveVendorId(index As Integer) As String
        Get
            Return m_CdDriveInfos(index).VendorId
        End Get
    End Property

    Public ReadOnly Property GetCdDriveProductId(index As Integer) As String
        Get
            Return m_CdDriveInfos(index).ProductId
        End Get
    End Property

    Public ReadOnly Property GetCdDriveProductRevision(index As Integer) As String
        Get
            Return m_CdDriveInfos(index).ProductRevision
        End Get
    End Property

    Public ReadOnly Property GetCdDriveVolumeName(index As Integer) As String
        Get
            Return m_CdDriveInfos(index).VolumeName
        End Get
    End Property

    Public ReadOnly Property GetCdDriveVolumePathNames(index As Integer) As List(Of String)
        Get
            Return m_CdDriveInfos(index).VolumePathNames
        End Get
    End Property

    Public Property RecorderUniqueId As String
        Get
            Return m_RecorderUniqueId
        End Get
        Set(value As String)
            m_RecorderUniqueId = value
        End Set
    End Property

    Public ReadOnly Property GetRecorderCanTrackAtOnce() As Boolean
        Get
            Return GetCanRecorderTrackAtOnce()
        End Get
    End Property

    Public ReadOnly Property GetRecorderIsCurrentMediaSupported() As Boolean
        Get
            Return GetRecorderCurrentMediaIsSupported()
        End Get
    End Property

    Public ReadOnly Property GetIsMediaPhysicallyBlank() As Boolean
        Get
            Return GetMediaPhysicallyBlank()
        End Get
    End Property

    Public ReadOnly Property GetIsMediaHeuristicallyBlank() As Boolean
        Get
            Return GetMediaHeuristicallyBlank()
        End Get
    End Property

    Public ReadOnly Property GetMediaTotalSectors() As Integer
        Get
            Return GetTotalSectorsOnMedia()
        End Get
    End Property

    Public ReadOnly Property GetMediaFreeSectors() As Integer
        Get
            Return GetFreeSectorsOnMedia()
        End Get
    End Property

    Public ReadOnly Property GetMediaUsedSectors() As Integer
        Get
            Return GetUsedSectorsOnMedia()
        End Get
    End Property
#End Region

#Region "Private Functions"
    Private Function GetCanRecorderTrackAtOnce() As Boolean
        Dim bolRet As Boolean = False
        If m_RecorderUniqueId <> String.Empty Then
            Dim pIDiscRecorder2 As IntPtr = CreateInterface(MsftDiscRecorder2, IID_IDiscRecorder2)
            If pIDiscRecorder2 <> IntPtr.Zero Then
                If DiscRecorder2_InitializeDiscRecorder(pIDiscRecorder2, m_RecorderUniqueId) Then
                    Dim pIDiscFormat2TrackAtOnce As IntPtr = CreateInterface(MsftDiscFormat2TrackAtOnce, IID_IDiscFormat2TrackAtOnce)
                    If pIDiscFormat2TrackAtOnce <> IntPtr.Zero Then
                        If DiscFormat2TrackAtOnce_PutRecorder(pIDiscFormat2TrackAtOnce, pIDiscRecorder2) Then
                            If DiscFormat2TrackAtOnce_PutClientName(pIDiscFormat2TrackAtOnce, "MyApplication") Then
                                bolRet = DiscFormat2_IsRecorderSupported(pIDiscFormat2TrackAtOnce, pIDiscRecorder2)
                            End If
                        End If
                        Marshal.Release(pIDiscFormat2TrackAtOnce)
                    End If
                End If
                Marshal.Release(pIDiscRecorder2)
            End If
        End If
        Return bolRet
    End Function

    Private Function GetRecorderCurrentMediaIsSupported() As Boolean
        Dim bolRet As Boolean = False
        If m_RecorderUniqueId <> String.Empty Then
            Dim pIDiscRecorder2 As IntPtr = CreateInterface(MsftDiscRecorder2, IID_IDiscRecorder2)
            If pIDiscRecorder2 <> IntPtr.Zero Then
                If DiscRecorder2_InitializeDiscRecorder(pIDiscRecorder2, m_RecorderUniqueId) Then
                    Dim pIDiscFormat2TrackAtOnce As IntPtr = CreateInterface(MsftDiscFormat2TrackAtOnce, IID_IDiscFormat2TrackAtOnce)
                    If pIDiscFormat2TrackAtOnce <> IntPtr.Zero Then
                        If DiscFormat2TrackAtOnce_PutRecorder(pIDiscFormat2TrackAtOnce, pIDiscRecorder2) Then
                            If DiscFormat2TrackAtOnce_PutClientName(pIDiscFormat2TrackAtOnce, "MyApplication") Then
                                bolRet = DiscFormat2_IsCurrentMediaSupported(pIDiscFormat2TrackAtOnce, pIDiscRecorder2)
                            End If
                        End If
                        Marshal.Release(pIDiscFormat2TrackAtOnce)
                    End If
                End If
                Marshal.Release(pIDiscRecorder2)
            End If
        End If
        Return bolRet
    End Function

    Private Function GetMediaPhysicallyBlank() As Boolean
        Dim bolRet As Boolean = False
        If m_RecorderUniqueId <> String.Empty Then
            Dim pIDiscRecorder2 As IntPtr = CreateInterface(MsftDiscRecorder2, IID_IDiscRecorder2)
            If pIDiscRecorder2 <> IntPtr.Zero Then
                If DiscRecorder2_InitializeDiscRecorder(pIDiscRecorder2, m_RecorderUniqueId) Then
                    Dim pIDiscFormat2TrackAtOnce As IntPtr = CreateInterface(MsftDiscFormat2TrackAtOnce, IID_IDiscFormat2TrackAtOnce)
                    If pIDiscFormat2TrackAtOnce <> IntPtr.Zero Then
                        If DiscFormat2TrackAtOnce_PutRecorder(pIDiscFormat2TrackAtOnce, pIDiscRecorder2) Then
                            If DiscFormat2TrackAtOnce_PutClientName(pIDiscFormat2TrackAtOnce, "MyApplication") Then
                                bolRet = DiscFormat2_GetMediaPhysicallyBlank(pIDiscFormat2TrackAtOnce)
                            End If
                        End If
                        Marshal.Release(pIDiscFormat2TrackAtOnce)
                    End If
                End If
                Marshal.Release(pIDiscRecorder2)
            End If
        End If
        Return bolRet
    End Function

    Private Function GetMediaHeuristicallyBlank() As Boolean
        Dim bolRet As Boolean = False
        If m_RecorderUniqueId <> String.Empty Then
            Dim pIDiscRecorder2 As IntPtr = CreateInterface(MsftDiscRecorder2, IID_IDiscRecorder2)
            If pIDiscRecorder2 <> IntPtr.Zero Then
                If DiscRecorder2_InitializeDiscRecorder(pIDiscRecorder2, m_RecorderUniqueId) Then
                    Dim pIDiscFormat2TrackAtOnce As IntPtr = CreateInterface(MsftDiscFormat2TrackAtOnce, IID_IDiscFormat2TrackAtOnce)
                    If pIDiscFormat2TrackAtOnce <> IntPtr.Zero Then
                        If DiscFormat2TrackAtOnce_PutRecorder(pIDiscFormat2TrackAtOnce, pIDiscRecorder2) Then
                            If DiscFormat2TrackAtOnce_PutClientName(pIDiscFormat2TrackAtOnce, "MyApplication") Then
                                bolRet = DiscFormat2_GetMediaHeuristicallyBlank(pIDiscFormat2TrackAtOnce)
                            End If
                        End If
                        Marshal.Release(pIDiscFormat2TrackAtOnce)
                    End If
                End If
                Marshal.Release(pIDiscRecorder2)
            End If
        End If
        Return bolRet
    End Function

    Private Function GetTotalSectorsOnMedia() As Integer
        Dim intRet As Integer = 0
        If m_RecorderUniqueId <> String.Empty Then
            Dim pIDiscRecorder2 As IntPtr = CreateInterface(MsftDiscRecorder2, IID_IDiscRecorder2)
            If pIDiscRecorder2 <> IntPtr.Zero Then
                If DiscRecorder2_InitializeDiscRecorder(pIDiscRecorder2, m_RecorderUniqueId) Then
                    Dim pIDiscFormat2TrackAtOnce As IntPtr = CreateInterface(MsftDiscFormat2TrackAtOnce, IID_IDiscFormat2TrackAtOnce)
                    If pIDiscFormat2TrackAtOnce <> IntPtr.Zero Then
                        If DiscFormat2TrackAtOnce_PutRecorder(pIDiscFormat2TrackAtOnce, pIDiscRecorder2) Then
                            If DiscFormat2TrackAtOnce_PutClientName(pIDiscFormat2TrackAtOnce, "MyApplication") Then
                                If DiscFormat2TrackAtOnce_PrepareMedia(pIDiscFormat2TrackAtOnce) Then
                                    intRet = DiscFormat2TrackAtOnce_GetTotalSectorsOnMedia(pIDiscFormat2TrackAtOnce)
                                    DiscFormat2TrackAtOnce_ReleaseMedia(pIDiscFormat2TrackAtOnce)
                                End If
                            End If
                        End If
                        Marshal.Release(pIDiscFormat2TrackAtOnce)
                    End If
                End If
                Marshal.Release(pIDiscRecorder2)
            End If
        End If
        Return intRet
    End Function

    Private Function GetFreeSectorsOnMedia() As Integer
        Dim intRet As Integer = 0
        If m_RecorderUniqueId <> String.Empty Then
            Dim pIDiscRecorder2 As IntPtr = CreateInterface(MsftDiscRecorder2, IID_IDiscRecorder2)
            If pIDiscRecorder2 <> IntPtr.Zero Then
                If DiscRecorder2_InitializeDiscRecorder(pIDiscRecorder2, m_RecorderUniqueId) Then
                    Dim pIDiscFormat2TrackAtOnce As IntPtr = CreateInterface(MsftDiscFormat2TrackAtOnce, IID_IDiscFormat2TrackAtOnce)
                    If pIDiscFormat2TrackAtOnce <> IntPtr.Zero Then
                        If DiscFormat2TrackAtOnce_PutRecorder(pIDiscFormat2TrackAtOnce, pIDiscRecorder2) Then
                            If DiscFormat2TrackAtOnce_PutClientName(pIDiscFormat2TrackAtOnce, "MyApplication") Then
                                If DiscFormat2TrackAtOnce_PrepareMedia(pIDiscFormat2TrackAtOnce) Then
                                    intRet = DiscFormat2TrackAtOnce_GetFreeSectorsOnMedia(pIDiscFormat2TrackAtOnce)
                                    DiscFormat2TrackAtOnce_ReleaseMedia(pIDiscFormat2TrackAtOnce)
                                End If
                            End If
                        End If
                        Marshal.Release(pIDiscFormat2TrackAtOnce)
                    End If
                End If
                Marshal.Release(pIDiscRecorder2)
            End If
        End If
        Return intRet
    End Function

    Private Function GetUsedSectorsOnMedia() As Integer
        Dim intRet As Integer = 0
        If m_RecorderUniqueId <> String.Empty Then
            Dim pIDiscRecorder2 As IntPtr = CreateInterface(MsftDiscRecorder2, IID_IDiscRecorder2)
            If pIDiscRecorder2 <> IntPtr.Zero Then
                If DiscRecorder2_InitializeDiscRecorder(pIDiscRecorder2, m_RecorderUniqueId) Then
                    Dim pIDiscFormat2TrackAtOnce As IntPtr = CreateInterface(MsftDiscFormat2TrackAtOnce, IID_IDiscFormat2TrackAtOnce)
                    If pIDiscFormat2TrackAtOnce <> IntPtr.Zero Then
                        If DiscFormat2TrackAtOnce_PutRecorder(pIDiscFormat2TrackAtOnce, pIDiscRecorder2) Then
                            If DiscFormat2TrackAtOnce_PutClientName(pIDiscFormat2TrackAtOnce, "MyApplication") Then
                                If DiscFormat2TrackAtOnce_PrepareMedia(pIDiscFormat2TrackAtOnce) Then
                                    intRet = DiscFormat2TrackAtOnce_GetUsedSectorsOnMedia(pIDiscFormat2TrackAtOnce)
                                    DiscFormat2TrackAtOnce_ReleaseMedia(pIDiscFormat2TrackAtOnce)
                                End If
                            End If
                        End If
                        Marshal.Release(pIDiscFormat2TrackAtOnce)
                    End If
                End If
                Marshal.Release(pIDiscRecorder2)
            End If
        End If
        Return intRet
    End Function

    Private Sub GetCdDriveInfos()
        m_CdDriveInfos.Clear()
        Dim pIDiscMaster2 As IntPtr = CreateInterface(MsftDiscMaster2, IID_IDiscMaster2)
        If pIDiscMaster2 <> IntPtr.Zero Then
            If DiscMaster2_GetIsSupportedEnvironment(pIDiscMaster2) Then
                Dim intCount As Integer = DiscMaster2_GetCount(pIDiscMaster2)
                If intCount > 0 Then
                    For intItem As Integer = 0 To intCount - 1
                        Dim tCdDriveInfo As New CdDriveInfo
                        Dim strUniqueID As String = DiscMaster2_GetItem(pIDiscMaster2, intItem)
                        If strUniqueID <> String.Empty Then
                            Dim pIDiscRecorder2 As IntPtr = CreateInterface(MsftDiscRecorder2, IID_IDiscRecorder2)
                            If pIDiscRecorder2 <> IntPtr.Zero Then
                                If DiscRecorder2_InitializeDiscRecorder(pIDiscRecorder2, strUniqueID) Then
                                    tCdDriveInfo.UniqueId = strUniqueID
                                    tCdDriveInfo.VendorId = DiscRecorder2_GetVendorId(pIDiscRecorder2)
                                    tCdDriveInfo.ProductId = DiscRecorder2_GetProductId(pIDiscRecorder2)
                                    tCdDriveInfo.ProductRevision = DiscRecorder2_GetProductRevision(pIDiscRecorder2)
                                    tCdDriveInfo.VolumeName = DiscRecorder2_GetVolumeName(pIDiscRecorder2)
                                    tCdDriveInfo.VolumePathNames = DiscRecorder2_GetVolumePathNames(pIDiscRecorder2)
                                    '.... was man sonst noch so an Infos vom CD-Laufwerk haben möchte
                                End If
                                Marshal.Release(pIDiscRecorder2)
                            End If
                        End If
                        m_CdDriveInfos.Add(tCdDriveInfo)
                    Next
                End If
            End If
            Marshal.Release(pIDiscMaster2)
        End If
    End Sub

    Private Function CreateInterface(CLSID As String, IID As String) As IntPtr
        CoCreateInstance(New Guid(CLSID), IntPtr.Zero, CLSCTX_INPROC_SERVER, New Guid(IID), CreateInterface)
    End Function

    Private Function GetMethodPtr(pInterface As IntPtr,
                                  intMethodNumber As Integer) As IntPtr
        If pInterface <> IntPtr.Zero And intMethodNumber >= 0 Then
            Dim pMethod As IntPtr() = New IntPtr(0) {}
            Marshal.Copy(IntPtr.Add(Marshal.ReadIntPtr(pInterface),
                         IntPtr.Size * intMethodNumber), pMethod, 0, 1)
            Return pMethod(0)
        End If
        Return IntPtr.Zero
    End Function
#End Region

#Region "Interface Delegates"
    Private Delegate Function IDiscMaster2_GetItem(<[In]> this As IntPtr,
                                                   <[In]> index As Integer,
                                                   <Out, MarshalAs(UnmanagedType.BStr)> ByRef value As String) As Integer

    Private Delegate Function IDiscMaster2_GetCount(<[In]> this As IntPtr,
                                                    <Out> ByRef value As Integer) As Integer

    Private Delegate Function IDiscMaster2_GetIsSupportedEnvironment(<[In]> this As IntPtr,
                                                                     <Out, MarshalAs(UnmanagedType.VariantBool)> ByRef value As Boolean) As Integer

    Private Delegate Function IDiscRecorder2_InitializeDiscRecorder(<[In]> this As IntPtr,
                                                                    <[In], MarshalAs(UnmanagedType.BStr)> recorderUniqueId As String) As Integer

    Private Delegate Function IDiscRecorder2_GetVendorId(<[In]> this As IntPtr,
                                                         <Out, MarshalAs(UnmanagedType.BStr)> ByRef value As String) As Integer

    Private Delegate Function IDiscRecorder2_GetProductId(<[In]> this As IntPtr,
                                                          <Out, MarshalAs(UnmanagedType.BStr)> ByRef value As String) As Integer

    Private Delegate Function IDiscRecorder2_GetProductRevision(<[In]> this As IntPtr,
                                                                <Out, MarshalAs(UnmanagedType.BStr)> ByRef value As String) As Integer

    Private Delegate Function IDiscRecorder2_GetVolumeName(<[In]> this As IntPtr,
                                                           <Out, MarshalAs(UnmanagedType.BStr)> ByRef value As String) As Integer

    Private Delegate Function IDiscRecorder2_GetVolumePathNames(<[In]> this As IntPtr,
                                                                <Out, MarshalAs(UnmanagedType.SafeArray, SafeArraySubType:=VarEnum.VT_VARIANT)> ByRef value As Object()) As Integer

    Private Delegate Function IDiscFormat2_IsRecorderSupported(<[In]> this As IntPtr,
                                                               <[In]> recorder As IntPtr,
                                                               <Out, MarshalAs(UnmanagedType.VariantBool)> ByRef value As Boolean) As Integer

    Private Delegate Function IDiscFormat2_IsCurrentMediaSupported(<[In]> this As IntPtr,
                                                                   <[In]> recorder As IntPtr,
                                                                   <Out, MarshalAs(UnmanagedType.VariantBool)> ByRef value As Boolean) As Integer

    Private Delegate Function IDiscFormat2_GetMediaPhysicallyBlank(<[In]> this As IntPtr,
                                                                   <Out, MarshalAs(UnmanagedType.VariantBool)> ByRef value As Boolean) As Integer

    Private Delegate Function IDiscFormat2_GetMediaHeuristicallyBlank(<[In]> this As IntPtr,
                                                                      <Out, MarshalAs(UnmanagedType.VariantBool)> ByRef value As Boolean) As Integer

    Private Delegate Function IDiscFormat2TrackAtOnce_PrepareMedia(<[In]> this As IntPtr) As Integer

    Private Delegate Function IDiscFormat2TrackAtOnce_ReleaseMedia(<[In]> this As IntPtr) As Integer

    Private Delegate Function IDiscFormat2TrackAtOnce_PutRecorder(<[In]> this As IntPtr,
                                                                  <[In]> recorder As IntPtr) As Integer

    Private Delegate Function IDiscFormat2TrackAtOnce_GetTotalSectorsOnMedia(<[In]> this As IntPtr,
                                                                             <Out> ByRef value As Integer) As Integer

    Private Delegate Function IDiscFormat2TrackAtOnce_GetFreeSectorsOnMedia(<[In]> this As IntPtr,
                                                                            <Out> ByRef value As Integer) As Integer

    Private Delegate Function IDiscFormat2TrackAtOnce_GetUsedSectorsOnMedia(<[In]> this As IntPtr,
                                                                            <Out> ByRef value As Integer) As Integer

    Private Delegate Function IDiscFormat2TrackAtOnce_PutClientName(<[In]> this As IntPtr,
                                                                    <[In], MarshalAs(UnmanagedType.BStr)> value As String) As Integer
#End Region

#Region "Interface Functions"
    Private Function DiscMaster2_GetItem(this As IntPtr, index As Integer) As String
        If this <> IntPtr.Zero Then
            Dim pMethodPtr As IntPtr = GetMethodPtr(this, VTable_Interfaces.GetItem)
            If pMethodPtr <> IntPtr.Zero Then
                Dim strValue As String = String.Empty
                If CType(Marshal.GetDelegateForFunctionPointer(pMethodPtr,
                         GetType(IDiscMaster2_GetItem)),
                         IDiscMaster2_GetItem)(this, index, strValue) = S_OK Then Return strValue


            End If
        End If
        Return String.Empty
    End Function

    Private Function DiscMaster2_GetCount(this As IntPtr) As Integer
        If this <> IntPtr.Zero Then
            Dim pMethodPtr As IntPtr = GetMethodPtr(this, VTable_Interfaces.GetCount)
            If pMethodPtr <> IntPtr.Zero Then
                Dim intValue As Integer = 0
                If CType(Marshal.GetDelegateForFunctionPointer(pMethodPtr,
                         GetType(IDiscMaster2_GetCount)),
                         IDiscMaster2_GetCount)(this, intValue) = S_OK Then Return intValue
            End If
        End If
        Return 0
    End Function

    Private Function DiscMaster2_GetIsSupportedEnvironment(this As IntPtr) As Boolean
        If this <> IntPtr.Zero Then
            Dim pMethodPtr As IntPtr = GetMethodPtr(this, VTable_Interfaces.GetIsSupportedEnvironment)
            If pMethodPtr <> IntPtr.Zero Then
                Dim bolValue As Boolean = False
                If CType(Marshal.GetDelegateForFunctionPointer(pMethodPtr,
                         GetType(IDiscMaster2_GetIsSupportedEnvironment)),
                         IDiscMaster2_GetIsSupportedEnvironment)(this, bolValue) = S_OK Then Return bolValue
            End If
        End If
        Return False
    End Function

    Private Function DiscRecorder2_InitializeDiscRecorder(this As IntPtr,
                                                          recorderUniqueId As String) As Boolean
        If this <> IntPtr.Zero Then
            Dim pMethodPtr As IntPtr = GetMethodPtr(this, VTable_Interfaces.InitializeDiscRecorder)
            If pMethodPtr <> IntPtr.Zero Then
                If CType(Marshal.GetDelegateForFunctionPointer(pMethodPtr,
                         GetType(IDiscRecorder2_InitializeDiscRecorder)),
                         IDiscRecorder2_InitializeDiscRecorder)(this, recorderUniqueId) = S_OK Then Return True
            End If
        End If
        Return False
    End Function

    Private Function DiscRecorder2_GetVendorId(this As IntPtr) As String
        If this <> IntPtr.Zero Then
            Dim pMethodPtr As IntPtr = GetMethodPtr(this, VTable_Interfaces.GetVendorId)
            If pMethodPtr <> IntPtr.Zero Then
                Dim strValue As String = String.Empty
                If CType(Marshal.GetDelegateForFunctionPointer(pMethodPtr,
                         GetType(IDiscRecorder2_GetVendorId)),
                         IDiscRecorder2_GetVendorId)(this, strValue) = S_OK Then Return strValue.Trim
            End If
        End If
        Return String.Empty
    End Function

    Private Function DiscRecorder2_GetProductId(this As IntPtr) As String
        If this <> IntPtr.Zero Then
            Dim pMethodPtr As IntPtr = GetMethodPtr(this, VTable_Interfaces.GetProductId)
            If pMethodPtr <> IntPtr.Zero Then
                Dim strValue As String = String.Empty
                If CType(Marshal.GetDelegateForFunctionPointer(pMethodPtr,
                         GetType(IDiscRecorder2_GetProductId)),
                         IDiscRecorder2_GetProductId)(this, strValue) = S_OK Then Return strValue.Trim
            End If
        End If
        Return String.Empty
    End Function

    Private Function DiscRecorder2_GetProductRevision(this As IntPtr) As String
        If this <> IntPtr.Zero Then
            Dim pMethodPtr As IntPtr = GetMethodPtr(this, VTable_Interfaces.GetProductRevision)
            If pMethodPtr <> IntPtr.Zero Then
                Dim strValue As String = String.Empty
                If CType(Marshal.GetDelegateForFunctionPointer(pMethodPtr,
                         GetType(IDiscRecorder2_GetProductRevision)),
                         IDiscRecorder2_GetProductRevision)(this, strValue) = S_OK Then Return strValue.Trim
            End If
        End If
        Return String.Empty
    End Function

    Private Function DiscRecorder2_GetVolumeName(this As IntPtr) As String
        If this <> IntPtr.Zero Then
            Dim pMethodPtr As IntPtr = GetMethodPtr(this, VTable_Interfaces.GetVolumeName)
            If pMethodPtr <> IntPtr.Zero Then
                Dim strValue As String = String.Empty
                If CType(Marshal.GetDelegateForFunctionPointer(pMethodPtr,
                         GetType(IDiscRecorder2_GetVolumeName)),
                         IDiscRecorder2_GetVolumeName)(this, strValue) = S_OK Then Return strValue.Trim
            End If
        End If
        Return String.Empty
    End Function

    Private Function DiscRecorder2_GetVolumePathNames(this As IntPtr) As List(Of String)
        Dim strList As New List(Of String)
        If this <> IntPtr.Zero Then
            Dim pMethodPtr As IntPtr = GetMethodPtr(this, VTable_Interfaces.GetVolumePathNames)
            If pMethodPtr <> IntPtr.Zero Then
                Dim objValue = New Object() {}
                If CType(Marshal.GetDelegateForFunctionPointer(pMethodPtr,
                         GetType(IDiscRecorder2_GetVolumePathNames)),
                         IDiscRecorder2_GetVolumePathNames)(this, objValue) = S_OK Then
                    Dim intCount As Integer = objValue.Length
                    If intCount > 0 Then
                        For intItem = 0 To intCount - 1
                            If objValue(intItem).GetType = GetType(String) Then
                                strList.Add(objValue(intItem).ToString.Trim)
                            End If
                        Next
                    End If
                    Return strList
                End If
            End If
        End If
        Return strList
    End Function

    Private Function DiscFormat2_IsRecorderSupported(this As IntPtr,
                                                     recorder As IntPtr) As Boolean
        If this <> IntPtr.Zero Then
            Dim pMethodPtr As IntPtr = GetMethodPtr(this, VTable_Interfaces.IsRecorderSupported)
            If pMethodPtr <> IntPtr.Zero Then
                Dim bolRet As Boolean = False
                If CType(Marshal.GetDelegateForFunctionPointer(pMethodPtr,
                         GetType(IDiscFormat2_IsRecorderSupported)),
                         IDiscFormat2_IsRecorderSupported)(this, recorder, bolRet) = S_OK Then Return bolRet
            End If
        End If
        Return False
    End Function

    Private Function DiscFormat2_IsCurrentMediaSupported(this As IntPtr,
                                                         recorder As IntPtr) As Boolean
        If this <> IntPtr.Zero Then
            Dim pMethodPtr As IntPtr = GetMethodPtr(this, VTable_Interfaces.IsCurrentMediaSupported)
            If pMethodPtr <> IntPtr.Zero Then
                Dim bolRet As Boolean = False
                If CType(Marshal.GetDelegateForFunctionPointer(pMethodPtr,
                         GetType(IDiscFormat2_IsCurrentMediaSupported)),
                         IDiscFormat2_IsCurrentMediaSupported)(this, recorder, bolRet) = S_OK Then Return bolRet
            End If
        End If
        Return False
    End Function

    Private Function DiscFormat2_GetMediaPhysicallyBlank(this As IntPtr) As Boolean
        If this <> IntPtr.Zero Then
            Dim pMethodPtr As IntPtr = GetMethodPtr(this, VTable_Interfaces.GetMediaPhysicallyBlank)
            If pMethodPtr <> IntPtr.Zero Then
                Dim bolRet As Boolean = False
                If CType(Marshal.GetDelegateForFunctionPointer(pMethodPtr,
                         GetType(IDiscFormat2_GetMediaPhysicallyBlank)),
                         IDiscFormat2_GetMediaPhysicallyBlank)(this, bolRet) = S_OK Then Return bolRet
            End If
        End If
        Return False
    End Function

    Private Function DiscFormat2_GetMediaHeuristicallyBlank(this As IntPtr) As Boolean
        If this <> IntPtr.Zero Then
            Dim pMethodPtr As IntPtr = GetMethodPtr(this, VTable_Interfaces.GetMediaHeuristicallyBlank)
            If pMethodPtr <> IntPtr.Zero Then
                Dim bolRet As Boolean = False
                If CType(Marshal.GetDelegateForFunctionPointer(pMethodPtr,
                         GetType(IDiscFormat2_GetMediaHeuristicallyBlank)),
                         IDiscFormat2_GetMediaHeuristicallyBlank)(this, bolRet) = S_OK Then Return bolRet
            End If
        End If
        Return False
    End Function

    Private Function DiscFormat2TrackAtOnce_PrepareMedia(this As IntPtr) As Boolean
        If this <> IntPtr.Zero Then
            Dim pMethodPtr As IntPtr = GetMethodPtr(this, VTable_Interfaces.PrepareMedia)
            If pMethodPtr <> IntPtr.Zero Then
                If CType(Marshal.GetDelegateForFunctionPointer(pMethodPtr,
                         GetType(IDiscFormat2TrackAtOnce_PrepareMedia)),
                         IDiscFormat2TrackAtOnce_PrepareMedia)(this) = S_OK Then Return True
            End If
        End If
        Return False
    End Function

    Private Function DiscFormat2TrackAtOnce_ReleaseMedia(this As IntPtr) As Boolean
        If this <> IntPtr.Zero Then
            Dim pMethodPtr As IntPtr = GetMethodPtr(this, VTable_Interfaces.ReleaseMedia)
            If pMethodPtr <> IntPtr.Zero Then
                If CType(Marshal.GetDelegateForFunctionPointer(pMethodPtr,
                         GetType(IDiscFormat2TrackAtOnce_ReleaseMedia)),
                         IDiscFormat2TrackAtOnce_ReleaseMedia)(this) = S_OK Then Return True
            End If
        End If
        Return False
    End Function

    Private Function DiscFormat2TrackAtOnce_PutRecorder(this As IntPtr,
                                                        recorder As IntPtr) As Boolean
        If this <> IntPtr.Zero Then
            Dim pMethodPtr As IntPtr = GetMethodPtr(this, VTable_Interfaces.PutRecorder)
            If pMethodPtr <> IntPtr.Zero Then
                If CType(Marshal.GetDelegateForFunctionPointer(pMethodPtr,
                         GetType(IDiscFormat2TrackAtOnce_PutRecorder)),
                         IDiscFormat2TrackAtOnce_PutRecorder)(this, recorder) = S_OK Then Return True
            End If
        End If
        Return False
    End Function

    Private Function DiscFormat2TrackAtOnce_GetTotalSectorsOnMedia(this As IntPtr) As Integer
        If this <> IntPtr.Zero Then
            Dim pMethodPtr As IntPtr = GetMethodPtr(this, VTable_Interfaces.GetTotalSectorsOnMedia)
            If pMethodPtr <> IntPtr.Zero Then
                Dim intRet As Integer = 0
                If CType(Marshal.GetDelegateForFunctionPointer(pMethodPtr,
                         GetType(IDiscFormat2TrackAtOnce_GetTotalSectorsOnMedia)),
                         IDiscFormat2TrackAtOnce_GetTotalSectorsOnMedia)(this, intRet) = S_OK Then Return intRet
            End If
        End If
        Return 0
    End Function

    Private Function DiscFormat2TrackAtOnce_GetFreeSectorsOnMedia(this As IntPtr) As Integer
        If this <> IntPtr.Zero Then
            Dim pMethodPtr As IntPtr = GetMethodPtr(this, VTable_Interfaces.GetFreeSectorsOnMedia)
            If pMethodPtr <> IntPtr.Zero Then
                Dim intRet As Integer = 0
                If CType(Marshal.GetDelegateForFunctionPointer(pMethodPtr,
                         GetType(IDiscFormat2TrackAtOnce_GetFreeSectorsOnMedia)),
                         IDiscFormat2TrackAtOnce_GetFreeSectorsOnMedia)(this, intRet) = S_OK Then Return intRet
            End If
        End If
        Return 0
    End Function

    Private Function DiscFormat2TrackAtOnce_GetUsedSectorsOnMedia(this As IntPtr) As Integer
        If this <> IntPtr.Zero Then
            Dim pMethodPtr As IntPtr = GetMethodPtr(this, VTable_Interfaces.GetUsedSectorsOnMedia)
            If pMethodPtr <> IntPtr.Zero Then
                Dim intRet As Integer = 0
                If CType(Marshal.GetDelegateForFunctionPointer(pMethodPtr,
                         GetType(IDiscFormat2TrackAtOnce_GetUsedSectorsOnMedia)),
                         IDiscFormat2TrackAtOnce_GetUsedSectorsOnMedia)(this, intRet) = S_OK Then Return intRet
            End If
        End If
        Return 0
    End Function

    Private Function DiscFormat2TrackAtOnce_PutClientName(this As IntPtr,
                                                          value As String) As Boolean
        If this <> IntPtr.Zero Then
            Dim pMethodPtr As IntPtr = GetMethodPtr(this, VTable_Interfaces.PutClientName)
            If pMethodPtr <> IntPtr.Zero Then
                If CType(Marshal.GetDelegateForFunctionPointer(pMethodPtr,
                         GetType(IDiscFormat2TrackAtOnce_PutClientName)),
                         IDiscFormat2TrackAtOnce_PutClientName)(this, value) = S_OK Then Return True
            End If
        End If
        Return False
    End Function
#End Region

End Class
